@@ -1,22 +1,26 @@
-# last modified 9 July 03 by J. Fox
+# last modified 29 November 03 by J. Fox
 
 .onAttach <- function(...){
-    cat("\nRcmdr Version 0.9-0\n")
+    cat("\nRcmdr Version 0.9-2\n")
     Commander()
     }
 
 .onLoad <- function(...){
     save.options <- options(warn=-1)
     on.exit(options(save.options))
+    lattice <- require(lattice)
     foreign <- require(foreign)
     mva <- require(mva)
     ctest <- require(ctest)
     tcltk <- require(tcltk)
+    abind <- require(abind)
+    effects <- require(effects)
     car <- require(car)
-    absent <- !c(foreign, mva, ctest, tcltk, car)
+    absent <- !c(lattice, foreign, mva, ctest, tcltk, abind, effects, car)
     if (any(absent)) {
         cat("\nThe following packages required by Rcmdr are missing:\n")
-        cat(paste(c("foreign", "mva", "ctest", "tcltk", "car")[absent], collapse=", "))
+        cat(paste(c("lattice", "foreign", "mva", "ctest", "tcltk", "abind", "effects", "car")[absent], 
+            collapse=", "))
         cat("\n")
         }
     }
